jQuery(function($) {
    var catalog = (function() {
        var $this;
        var categoryParams = [];
        var existsCategories = [];
        var page = 1;
        var modalForm = '';
        var categorySlug = '';

        return {
            init: function(el) {
                if ($this) {
                    return ;
                }

                $this = this;
                $this.initPagination();
                $('body').on(
                    'page:load',
                    function(e) {
                        if (typeof e.action != 'undefined' && e.action) {
                            $('.modal').modal('hide');
                            $this.mainRequest($('#category-list'), page);
                            return false;
                        }
                        if (typeof e.response != 'undefined'
                            && e.response.hasOwnProperty('categoriesExists')
                        ) {
                            existsCategories = e.response.categoriesExists;
                        }
                        $this.initModal();
                        $this.initAddButton(e.obj);
                        $this.initEditButtion(e.obj);
                        $this.initDeleteButton(e.obj);
                        $this.initEditAttributeButtion(e.obj);
                        $this.changeEnable(e.obj);
                        $this.initResetButton(e.obj);
                        $this.initSelect(e.obj, '#google_category_select');
                        $this.initTooltip(e.obj);
                    }
                );
            },
            initPagination: function() {
                $('body').on(
                    'pagination:click',
                    $('.pagination-menu'),
                    function(e) {
                        $this.mainRequest(e.obj, e.page);
                    }
                );
            },
            mainRequest: function ($el, page) {
                var $this = this;
                var th = $el,
                        tmpl = $(th.data('ads_template')).html(),
                        action = th.data('addon_action'),
                        target = $(th.data('ads_target')),
                        data = {},
                        fields = $el.find('.ads-field');

                if (fields.length > 0) {
                    data = fields.serialize();
                } else {
                    if (typeof page == 'undefined') {
                        page = 1;
                    }
                    data = 'page=' + page;
                }

                if (categorySlug != '') {
                    data += '&category_slug=' + categorySlug;
                }

                return new Promise(function(resolve, reject) {
                    $.ajax({
                        url: ajaxurl,
                        data: {
                            action: 'googlecatalog_ajax',
                            ads_action: action,
                            args: data
                        },
                        type: "POST",
                        dataType: 'json',
                        success: function (response) {
                            if (response) {
                                if (response.hasOwnProperty('error')) {
                                    $this.notify(response.error, 'danger');
                                    resolve(false);
                                } else {
                                    resolve(true);
                                    target.html($this.objTotmpl(tmpl, response));
                                    $this.createPagination(target, response.totalPage, response.page, response.limit);
                                    if (response.hasOwnProperty('category_select')) {
                                        $this.fillSelectData($('[name="category_select"]'), response.category_select);
                                    }
                                    if (response.hasOwnProperty('categories')) {
                                        $this.fillSelectData($('[name="google_category_select[]"]'), response.categories);
                                    }
                                    if (response.hasOwnProperty('categoriesExists')) {
                                        existsCategories = response.categoriesExists;
                                        console.log(existsCategories);
                                    }
                                    $(target).trigger({
                                        type : "page:load",
                                        obj  : target
                                    });
                                }
                            }
                        }
                    });
                });
            },
            objTotmpl: function (tmpl, data) {
                if (typeof Handlebars === 'undefined') {
                    console.log('Handlebars not registry');
                    return false
                }
                var template = Handlebars.compile(tmpl);
                return template(data);
            },
            notify: function(message, type) {
                window.Notify(message, null, null, type);
            },
            initModal: function() {
                if (modalForm == '') {
                    modalForm = $('.modal-wrapper').html();
                    $('.edit-category-form').remove();
                    $('body').append(modalForm);
                }
            },
            createPagination: function (obj, total, current, perPage) {
                perPage = typeof perPage !== 'undefined' ? perPage : 10;
                var $obj = $(obj).find('.pagination-menu');
                $obj.pagination({
                    items: parseInt(total),
                    itemsOnPage: parseInt(perPage),
                    currentPage: parseInt(current),
                    cssStyle: "light-theme",
                    prevText: '<span class="fa fa-angle-left"></span>',
                    nextText: '<span class="fa fa-angle-right"></span>',
                    onPageClick: function (pageNumber) {
                        page = pageNumber;
                        $.event.trigger({
                            type: "pagination:click",
                            obj: obj,
                            page: pageNumber
                        });

                        $(obj).trigger({
                            type : "pagination:click",
                            obj  : obj,
                            page : pageNumber
                        });
                    }
                });
            },
            switchery: function(el) {
                var elems = el.find('.switchery');
                for (var i = 0; i < elems.length; i++) {
                    var switchery = new Switchery(elems[i]);
                }
            },
            initAddButton: function(el) {
                el.find('#add-category').on('click', function(e) {
                    var that = $(this);
                    e.preventDefault();
                    e.stopPropagation();
                    $('.edit-category-form').find('.panel-title').html('Add Category');
                    categorySlug = '';
                    $this.mainRequest($('#category-form')).then(function(isShow) {
                        if (isShow) {
                            $('.edit-category-form').modal('show');
                            $this.switchery($('.edit-category-form'));
                        }
                    });
                });
            },
            initEditButtion: function(el) {
                el.find('.edit-category').on('click', function(e) {
                    var that = $(this);
                    e.preventDefault();
                    e.stopPropagation();
                    categorySlug = that.data('category-slug');
                    $('.edit-category-form').find('.panel-title').html('Edit Category');
                    $this.mainRequest($('#category-form')).then(function(isShow) {
                        if (isShow) {
                            $('.edit-category-form').modal('show');
                            $this.switchery($('.edit-category-form'));
                        }
                    });
                });
            },
            initEditAttributeButtion: function(el) {
                $(document).off('click', '.edit-attribute');
                $(document).on('click', '.edit-attribute', function(e) {
                    var that = $(this);
                    e.preventDefault();
                    e.stopPropagation();
                    var attributeName = that.data('attribute');
                    var field = $('[data-id="' + attributeName + '"]');
                    var column = field.find('.edit-field-attribute');
                    var fieldValue = column.find('p').html();
                    var newTextArea = $('<div style="display:inline-block;width:90%;">\n\
                        <textarea rows="4" cols="4" class="form-control"\n\
                            style="background:#fff;border: 1px solid #5b9dd9;"\n\
                            name="save-attribute"\n\
                            value="' + fieldValue + '" /></textarea>');
                    that.removeClass('pencil-btn').addClass('save-btn');
                    that.find('i').removeClass('fa-pencil').addClass('fa-floppy-o');
                    if (that.hasClass('edit-attribute')) {
                        that.removeClass('edit-attribute').addClass('save-attribute');
                    }

                    column.html('').append(newTextArea);
                    column.find('textarea[name="save-attribute"]').val(fieldValue);
                });
                $(document).off('click', '.save-attribute');
                $(document).on('click', '.save-attribute', function(e) {
                    var that = $(this);
                    e.preventDefault();
                    e.stopPropagation();
                    var attributeName = that.data('attribute');
                    var field = $('[data-id="' + attributeName + '"]');
                    var column = field.find('textarea[name="save-attribute"]').parents('div:first');
                    var fieldValue = field.find('textarea[name="save-attribute"]').val();
                    that.addClass('animate-spinner').prop('disabled', true);

                    $.ajax({
                        url: ajaxurl,
                        data: {
                            action: 'googlecatalog_ajax',
                            ads_action: 'save_attribute',
                            args: $.param({
                                attribute: attributeName,
                                values: fieldValue
                            })
                        },
                        type: "POST",
                        dataType: 'json',
                        success: function(response) {
                            that.removeClass('animate-spinner').prop('disabled', false);
                            if (response.result) {
                                that.removeClass('save-btn').addClass('pencil-btn');
                                that.find('i').removeClass('fa-floppy-o').addClass('fa-pencil');
                                if (that.hasClass('save-attribute')) {
                                    that.removeClass('save-attribute').addClass('edit-attribute');
                                    column.html('');
                                    column.append('<p>' + fieldValue + '</p>');
                                }
                                $this.notify(response.message, 'success');
                            } else {
                                $this.notify(response.message, 'danger');
                            }
                        }
                    });
                });
            },
            initDeleteButton: function(el) {
                var deleteButton = el.find('.delete-category');
                deleteButton.off('click');
                deleteButton.on('click', function(e) {
                    var that = $(this);
                    e.preventDefault();
                    e.stopPropagation();
                    var that = $(this);
                    var params = {
                        'slug' : that.data('category-slug')
                    };

                    bootbox.confirm({
                        message: 'Are you sure you want to delete?',
                        buttons: {
                            confirm: {
                                label: 'Yes',
                                className: 'btn-danger'
                            },
                            cancel: {
                                label: 'Cancel',
                                className: 'btn-blue-trasparent'
                            }
                        },
                        callback: function(result) {
                            if (result === true) {
                                $.ajax({
                                    url: ajaxurl,
                                    data: {
                                        action: 'googlecatalog_ajax',
                                        ads_action: 'delete_category',
                                        args: $.param(params)
                                    },
                                    type: 'POST',
                                    dataType: 'json',
                                    success: function(response) {
                                        if (!response) {
                                            return false;
                                        }

                                        if (!response.result) {
                                            $this.notify(response.message, 'danger');
                                        } else {
                                            $this.mainRequest($('#category-list'), page);
                                            $this.notify(response.message, 'success');
                                        }
                                    }
                                });
                            }
                        }
                    });
                });
            },
            initSelect: function(el, id) {
                console.log(existsCategories);
                $(el.find('[name="google_category_select[]"]')).selectpicker('refresh');
                $(el.find('[name="category_select"]')).find('option').each(function(index, value) {
                    if (existsCategories.includes($(value).val())) {
                        $(value).attr('data-icon', 'fa fa-save before-padding-left-5');
                        $(value).data('icon', 'fa fa-save before-padding-left-5');
                    } else {
                        $(value).removeAttr('data-icon');
                        $(value).data('icon', null);
                    }
                });
                $(el.find('[name="category_select"]')).selectpicker('refresh');

                var googleCategory = el.find(id);
                //googleCategory.unbind('change');
                googleCategory.on(
                    'change',
                    function(e) {
                        e.preventDefault();
                        e.stopPropagation();
                        var that = $(this);
                        el.find('[name="google_category_select[]"]').each(function(index, value) {
                            var currentId = that.prop('id').split('-')[1];
                            var tempId = $(value).prop('id').split('-')[1];
                            if (typeof currentId == 'undefined' && typeof tempId != 'undefined') {
                                $(value).parents('.form-group').remove();
                            }

                            if (typeof currentId != 'undefined' && currentId < tempId) {
                                $(value).parents('.form-group').remove();
                            }
                        });

                        var count = el.find('[name="google_category_select[]"]').length;
                        var idName = 'google_category_select';
                        var cloneGoogleSelect = that.clone()
                                                    .attr('data-parent', that.prop('id'))
                                                    .data('parent', that.prop('id'))
                                                    .prop('id', idName + '-' + count);

                        $.ajax({
                            url: ajaxurl,
                            type: "post",
                            data: {
                                action: 'googlecatalog_ajax',
                                ads_action: 'category_children',
                                args: 'id=' + that.val()
                            },
                            dataType: 'json',
                            success: function(xhr) {
                                if (xhr.result != true) {
                                    return ;
                                }

                                if (xhr.categories.length == 0) {
                                    return ;
                                }

                                if (el.find('#google_category_select-' + count).length > 0) {
                                    return ;
                                }

                                cloneGoogleSelect.find('option').each(
                                    function(index, value) {
                                        $(value).remove();
                                    }
                                );

                                $.each(xhr.categories, function(index, value) {
                                    cloneGoogleSelect.append(
                                        $("<option></option>").attr("value", value.id)
                                                              .text(value.name)
                                    );
                                });

                                that.parents('.form-group')
                                    .after('<div class="form-group"></div>');
                                that.parents('.form-group').next().append(cloneGoogleSelect);
                                $this.initSelect(el, '#google_category_select-' + count);
                            }
                        });
                    }
                );
            },
            initTooltip: function(el) {
                $('[data-toggle="tooltip"]').tooltip();
            },
            changeEnable: function(el) {
                var enableButton = el.find('[name="enable"]');
                enableButton.on('change', function() {
                    if (!$(this).is(':checked')) {
                        $('#document-download-xml').hide();
                        $('#document-copy-url').hide();
                        $('#document-download-xml').parents('.inline-buttons').addClass('hidden-block');
                    } else {
                        $('#document-download-xml').show();
                        $('#document-copy-url').show();
                        $('#document-download-xml').parents('.inline-buttons').removeClass('hidden-block');
                    }
                });
            },
            initResetButton: function(el) {
                var resetButton = el.find('#reset_google_attribute');
                resetButton.off('click');
                resetButton.on('click', function(e) {
                    e.preventDefault();
                    e.stopPropagation();
                    var that = $(this);
                    that.addClass('animate-spinner');
                    $.ajax({
                        url: ajaxurl,
                        type: 'POST',
                        data: {
                            action: 'googlecatalog_ajax',
                            ads_action: 'reset_attributes',
                            args: $.param({
                                'reset' : true 
                            })
                        },
                        dataType: 'json',
                        success: function(response) {
                            that.removeClass('animate-spinner');
                            if (response.result) {
                                $this.notify(response.message, 'success');
                                $this.mainRequest($('#attributes-list'));
                            } else {
                                $this.notify(response.message, 'danger');
                            }
                        }
                    });
                });
            },
            fillSelectData: function(el, response) {
                if (typeof response == 'string') {
                    el.val(response);
                }

                if (typeof response == 'object') {
                    var googleCategorySelect = $('#google_category_select');
                    var propId = googleCategorySelect.prop('id');
                    googleCategorySelect.val(response[0].id);
                    $.each(response, function(index, value) {
                        if (index >= 1) {
                            var count = $('[name="google_category_select[]"]').length;
                            var idName = 'google_category_select';
                            var googleCategorySelectClone = googleCategorySelect.clone();
                            googleCategorySelectClone.val(value.id);
                            googleCategorySelectClone.find('option').each(
                                function(index, value) {
                                    $(value).remove();
                                }
                            );

                            $.each(value.options, function(index, option) {
                                googleCategorySelectClone.append(
                                    $("<option></option>").attr("value", option.id)
                                                          .text(option.name)
                                );
                            });

                            googleCategorySelectClone.val(value.id);
                            googleCategorySelectClone.attr('data-parent', propId)
                                                     .data('parent', propId)
                                                     .prop('id', idName + '-' + count);
                            propId = idName + '-' + count;
                            googleCategorySelect.parents('.form-group')
                                                .after('<div class="form-group"></div>');
                            googleCategorySelect.parents('.form-group').next().append(googleCategorySelectClone);
                            googleCategorySelect = $('[name="google_category_select[]"]').last();
                        }
                    });
                }
            }
        }
    })();

    catalog.init();
});