<?php

namespace progress\models;
use progress\models\pdo\MysqlPDO as MysqlPDO;

/**
 * Description of PostMeta
 *
 * @author Artem Yuriev art3mk4@gmail.com Dec 18, 2018 11:07:50 AM
 */
class PostMeta extends MysqlPDO
{

    /**
     *
     * @var type 
     */
    private static $instance;

    /**
     * PostMeta construct
     */
    public function __construct()
    {
        $this->setTable('postmeta');
        parent::__construct();
    }

    /**
     * instance
     * 
     * @return \progress\models\PostMeta
     */
    public static function instance()
    {
        if (!isset(self::$instance)) {
            self::$instance = new PostMeta();
        }

        return self::$instance;
    }

    /**
     * findByPostId
     * 
     * @param type $post_id
     * @return type
     */
    public function findByPostId($post_id)
    {
        $result = $this->findAll(
            array(
                'post_id' => $post_id
            )
        );

        if (!$result) {
            return array();
        }

        return $result;
    }

    /**
     * getAttributesByPostId
     * 
     * @param type $post_id
     */
    public function getAttributesByPostId($post_id)
    {
        $sql = "SELECT * FROM " . TABLE_PREFIX . $this->table . " WHERE post_id=:post_id AND meta_key like 'attribute_%'";
        $select = $this->prepare($sql);
        $select->bindValue(':post_id', $post_id);
        $select->execute();
        $result = $select->fetchAll(\PDO::FETCH_ASSOC);
        if (!$result) {
            return array();
        }

        return $result;
    }

    /**
     * getPriceByPostId
     * 
     * @param type $post_id
     * @return type
     */
    public function getPriceByPostId($post_id)
    {
        $sql = "SELECT MIN(meta_value) as price FROM " . TABLE_PREFIX . $this->table . " WHERE post_id=:post_id AND meta_key IN ('_price', '_regular_price', '_sale_price') AND meta_value != 0";
        $select = $this->prepare($sql);
        $select->bindValue(':post_id', $post_id);
        $select->execute();
        $result = $select->fetch(\PDO::FETCH_ASSOC);
        if (!isset($result['price'])) {
            return false;
        }

        return $result['price'];
    }

    /**
     * getImageByPostId
     * 
     * @param type $post_id
     * @return type
     */
    public function getImageByPostId($post_id)
    {
        $sql = "SELECT meta.meta_value FROM " . TABLE_PREFIX . $this->table . " meta
            INNER JOIN " . TABLE_PREFIX . $this->table . " joinMeta ON meta.post_id = joinMeta.meta_value
            WHERE joinMeta.meta_key = '_thumbnail_id' AND meta.meta_key = '_wp_attached_file' AND joinMeta.post_id = :post_id";
        $select = $this->prepare($sql);
        $select->bindValue(':post_id', $post_id);
        $select->execute();
        $result = $select->fetch(\PDO::FETCH_ASSOC);
        if (!$result) {
            return false;
        }

        $fileName = ABSPATH . DIRECTORY_SEPARATOR . 'uploads' . DIRECTORY_SEPARATOR . $result['meta_value'];
        if (file_exists($fileName)) {
            return ADS_FACEBOOK_SITEURL . '/wp-content/uploads/' .  $result['meta_value'];
        }

        return $result['meta_value'];
    }

    /**
     * getProductAttributesByPostId
     * 
     * @param type $post_id
     * @return boolean
     */
    public function getProductAttributesByPostId($post_id)
    {
        $sql = "SELECT meta_value FROM " . TABLE_PREFIX . $this->table . " WHERE post_id=:post_id AND meta_key = '_product_attributes'";
        $select = $this->prepare($sql);
        $select->bindValue(':post_id', $post_id);
        $select->execute();
        $result = $select->fetch(\PDO::FETCH_ASSOC);
        if (!isset($result['meta_value'])) {
            return false;
        }

        return $result['meta_value'];
    }
}