<?php

namespace progress\models;
use progress\models\pdo\MysqlPDO as MysqlPDO;

/**
 * Description of Settings
 *
 * @author Artem Yuriev art3mk4@gmail.com Nov 30, 2018 5:47:39 PM
 */
class Settings extends MysqlPDO
{

    /**
     *
     * @var type 
     */
    private static $instance;

    /**
     * Settings construct
     */
    public function __construct()
    {
        $this->setTable('options');
        parent::__construct();
    }

    /**
     * instance
     * 
     * @return \progress\models\Settings
     */
    public static function instance()
    {
        if (!isset(self::$instance)) {
            self::$instance = new Settings();
        }

        return self::$instance;
    }

    /**
     * init
     */
    public static function init()
    {
        $settings = Settings::instance();
        $options = $settings->getAdsOptions();
        $getAdsWooCurrency = $settings->getAdsWooCurrency();
        $siteurl = $settings->getSiteUrlOptions();
        $blogname = $settings->getBlogName();
        $blogDescription = $settings->getBlogDescription();
        $blogUrl = $settings->getBlogUrl();
        if (!defined('ADS_PRICE_ROUNDING')) define('ADS_PRICE_ROUNDING', 0);
        if (!defined('ADS_PRICE_ASSIGNCENTS')) define('ADS_PRICE_ASSIGNCENTS', '');
        if (!defined('ADS_FACEBOOK_SITEURL')) define('ADS_FACEBOOK_SITEURL', $siteurl);
        if (!defined('WP_ADS_BLOGNAME')) define('WP_ADS_BLOGNAME', $blogname);
        if (!defined('WP_ADS_BLOGDESCRIPTION')) define('WP_ADS_BLOGDESCRIPTION', $blogDescription);
        if (!defined('WP_ADS_BLOGURL')) define('WP_ADS_BLOGURL', $blogUrl);

        $defaults = [
            'AppKey'       => '',
            'trackingId'   => '',
            'affiliateUrl' => '',
            'language'     => 'en',
            'country'      => 'US',
            'currency'     => [
                'currency'    => 'USD',
                'paycurrency' => 'USD',
                'curlist'     => ['USD'],
                'value'       => ['USD' => 1],
                'rounding'    => '0',
                'assignCents' => ''
            ]
        ];

        if (!$options) $options = [];
        $params = Helper::merge_defaults($options, $defaults);
        if (!defined('ADS_CUVALUE')) define('ADS_CUVALUE', serialize($params['currency']['value']));
        if (!defined('ADSW_CUVALUE')) define('ADSW_CUVALUE', $getAdsWooCurrency); 
    }

    /**
     * getOptions
     */
    public function getFacebookOptions()
    {
        require_once  ADSFBCATALOG_PATH . '/libs/models/fbcatalog/General.php';
        $result = $this->find(
            array(
                'option_name' => \models\fbcatalog\General::NAME
            )
        );

        if (isset($result['option_value'])) {
            return self::maybe_unserialize($result['option_value']);
        }

        return false;
    }

    /**
     * getAdsOptions
     */
    public function getAdsOptions()
    {
        $result = $this->find(
            array(
                'option_name' => 'ads-settings'
            )
        );

        if (isset($result['option_value'])) {
            return self::maybe_unserialize($result['option_value']);
        }

        return false;
    }

    /**
     * getAdsWooCurrency
     * 
     * @return boolean
     */
    public function getAdsWooCurrency()
    {
        $result = $this->find(
            array(
                'option_name' => 'adsw_currency_settings'
            )
        );

        if (isset($result['option_value'])) {
            return self::maybe_unserialize($result['option_value']);
        }

        return false;
    }

    /**
     * getSiteUrlOptions
     * 
     * @return boolean
     */
    public function getSiteUrlOptions()
    {
        $result = $this->find(
            array(
                'option_name' => 'siteurl'
            )
        );

        if (isset($result['option_value'])) {
            return self::maybe_unserialize($result['option_value']);
        }

        return false;
    }

    /**
     * getBlogName
     * 
     * @return boolean
     */
    public function getBlogName()
    {
        $result = $this->find(
            array(
                'option_name' => 'blogname'
            )
        );

        if (isset($result['option_value'])) {
            return $result['option_value'];
        }

        return false;
    }

    /**
     * getBlogDescription
     * 
     * @return boolean
     */
    public function getBlogDescription()
    {
        $result = $this->find(
            array(
                'option_name' => 'blogdescription'
            )
        );

        if (isset($result['option_value'])) {
            return $result['option_value'];
        }

        return false;
    }

    /**
     * getBlogUrl
     * 
     * @return boolean
     */
    public function getBlogUrl()
    {
        $result = $this->find(
            array(
                'option_name' => 'home'
            )
        );

        if (isset($result['option_value'])) {
            return $result['option_value'];
        }

        return false;
    }

    /**
     * getPermalinkStructure
     * 
     * @return boolean
     */
    public function getPermalinkStructure()
    {
        $result = $this->find(
            array(
                'option_name' => 'permalink_structure'
            )
        );

        if (isset($result['option_value'])) {
            return $result['option_value'];
        }

        return false;
    }

    /**
     * getWoocommerceCurrencyPos
     * 
     * @return boolean
     */
    public function getWoocommerceCurrencyPos()
    {
        $result = $this->find(
            array(
                'option_name' => 'woocommerce_currency_pos'
            )
        );

        if (isset($result['option_value'])) {
            return $result['option_value'];
        }

        return false;
    }

    /**
     * getDefaultCurrency
     * 
     * @return boolean
     */
    public function getDefaultCurrency()
    {
        $result = $this->find(
            array(
                'option_name' => 'woocommerce_currency'
            )
        );

        if (isset($result['option_value'])) {
            return $result['option_value'];
        }

        return false;
    }

    /**
     * maybe_unserialize
     * 
     * @param type $value
     * @return type
     */
    public static function maybe_unserialize($value)
    {
        if (self::is_serialized($value)) {
            $value = unserialize($value);
            return self::maybe_unserialize($value);
        }

        return $value;
    }

    /**
     * is_serialized
     * 
     * @param type $data
     * @return boolean
     */
    public static function is_serialized($data, $strict = true)
    {
	// if it isn't a string, it isn't serialized.
	if (!is_string($data)) {
            return false;
	}

	$data = trim($data);
 	if ('N;' == $data) {
            return true;
	}

	if (strlen($data) < 4) {
            return false;
	}

	if (':' !== $data[1]) {
            return false;
	}

	if ($strict) {
            $lastc = substr($data, -1 );
            if (';' !== $lastc
                && '}' !== $lastc
            ) {
                return false;
            }
	} else {
            $semicolon = strpos($data, ';');
            $brace     = strpos($data, '}');
            // Either ; or } must exist.
            if (false === $semicolon && false === $brace)
                return false;
            // But neither must be in the first X characters.
            if (false !== $semicolon && $semicolon < 3)
                return false;
            if (false !== $brace && $brace < 4)
                return false;
	}
	$token = $data[0];
	switch ($token) {
            case 's' :
                if ($strict) {
                    if ('"' !== substr($data, -2, 1)) {
                        return false;
                    }
                } elseif (false === strpos($data, '"')) {
                    return false;
                }
                    // or else fall through
            case 'a' :
            case 'O' :
                return (bool)preg_match("/^{$token}:[0-9]+:/s", $data);
            case 'b' :
            case 'i' :
            case 'd' :
                $end = $strict ? '$' : '';
                return (bool)preg_match("/^{$token}:[0-9.E-]+;$end/", $data);
	}
	return false;
    }
}