<?php

namespace progress\models;
use progress\models\pdo\MysqlPDO as MysqlPDO;

/**
 * Description of WooProduct
 *
 * @author Artem Yuriev art3mk4@gmail.com Feb 15, 2019 11:02:02 AM
 */
class WooProduct extends Product
{

    /**
     *
     * @var type 
     */
    public $currency_pos;

    /**
     *
     * @var type 
     */
    public $default_currency;

        /**
     * constructor WooProduct
     */
    public function __construct()
    {
        $settings = Settings::instance();
        $this->currency_pos = $settings->getWoocommerceCurrencyPos();
        $this->default_currency = $settings->getDefaultCurrency();
        $this->xmlGenerator = new XmlGenerator();
        $this->setTable('posts');
        MysqlPDO::__construct();
    }

    /**
     * getTotalProducts
     * 
     * @param type $category_slug
     * @return type
     */
    public function getTotalProducts($category_slug = '')
    {
        $selectSql = "SELECT count(DISTINCT posts.id) as count FROM " . TABLE_PREFIX . $this->table . " posts
                    LEFT JOIN " . TABLE_PREFIX . "term_relationships tr ON (posts.ID = tr.object_id)
                    LEFT JOIN " . TABLE_PREFIX . "term_taxonomy tt ON (tr.term_taxonomy_id = tt.term_taxonomy_id)
                    LEFT JOIN " . TABLE_PREFIX . "terms t ON (tt.term_id = t.term_id)
                    WHERE posts.post_status = 'publish' AND posts.post_type = 'product'";

        if (is_array($category_slug)) {
            $categoryString = implode("', '", $category_slug);
        } else {
            $categoryString = $category_slug;
        }

        if (!empty($categoryString)) {
            $selectSql .= " AND tt.taxonomy = 'product_cat'
                    AND t.slug in ('{$categoryString}')";
        } else {
            $selectSql .= " AND tt.taxonomy = 'product_cat' AND tt.parent = 0";
        }

        $selectSql .= " GROUP BY posts.ID, posts.post_title";
        $selectSql = "SELECT count(*) as count FROM ({$selectSql}) as x";
        $select = $this->prepare($selectSql);
        $select->execute();
        return $select->fetch(\PDO::FETCH_ASSOC);
    }

    /**
     * findAll
     * 
     * @param type $page
     * @param type $category_slug
     * @param type $currency
     */
    public function findAll($page = 1, $category_slug = '', $currency = '')
    {
        $limit = 1;
        $start = ($page - 1) * $limit;
        $sql = "SELECT posts.ID as id,
                posts.post_title as title,
                posts.post_content as description,
                t.name as google_product_category,
                relation.google_category_text as alias_category,
                (SELECT MAX(identifier_exists)
                    FROM " . TABLE_PREFIX . "ads_googlemerchant_category_relation relation
                    WHERE relation.term_id = t.term_id
                ) identifier_exists,
                t.term_id as category_id,
                alimeta.product_id as mpn
                FROM " . TABLE_PREFIX . $this->table . " posts
                LEFT JOIN " . TABLE_PREFIX . "postmeta pmeta ON pmeta.post_id = posts.ID
                LEFT JOIN " . TABLE_PREFIX . "adsw_ali_meta alimeta ON alimeta.post_id = posts.ID
                LEFT JOIN " . TABLE_PREFIX . "term_relationships tr ON posts.ID = tr.object_id
                LEFT JOIN " . TABLE_PREFIX . "term_taxonomy tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
                LEFT JOIN " . TABLE_PREFIX . "terms t ON tt.term_id = t.term_id
                LEFT JOIN " . TABLE_PREFIX . "ads_googlemerchant_category_relation relation ON relation.term_id = t.term_id
                WHERE posts.post_type = 'product'
                AND posts.post_status = 'publish'";

        if (is_array($category_slug)) {
            $categoryString = implode("', '", $category_slug);
        } else {
            $categoryString = $category_slug;
        }

        if (!empty($categoryString)) {
            $sql .= " AND tt.taxonomy = 'product_cat'
                    AND t.slug in ('{$categoryString}') ";
            $parent = true;
        } else {
            $sql .= " AND tt.taxonomy = 'product_cat' AND tt.parent = 0 ";
            $parent = false;
        }

        $sql .= "GROUP BY posts.ID, posts.post_title LIMIT :start, :limit";
        $select = $this->prepare($sql);
        $select->bindValue(':start', $start, \PDO::PARAM_INT);
        $select->bindValue(':limit', $limit, \PDO::PARAM_INT);
        $select->execute();
        $items = $select->fetchAll(\PDO::FETCH_ASSOC);
        if (!$items) {
            return false;
        }
        foreach ($items as $value) {
            //$value['id'] = 352;//2164;1045
            $params = $this->getProductVariations($value['id']);
            $value['brand'] = WP_ADS_BLOGNAME;
            $value['google_product_category'] = $this->getGoogleProductCategory($value, $parent);
            $result = $this->getItem($value);
            if ($result == false) {
                continue;
            }

            $result['mpn'] = $value['mpn'];
            if (!isset($value['identifier_exists'])) {
                $result['identifier_exists'] = 0;
            } else {
                $result['identifier_exists'] = $value['identifier_exists'];
            }
            $result['availability'] = 'in stock';
            $result['position'] = $this->currency_pos;
            $result['default_currency'] = $this->default_currency;
            $result['select_currency'] = $currency;
            $result['price'] = $this->getWooPrice($result, $currency);
            $this->multiplyProduct($result, $params);
        }
    }

    /**
     * findVariationsByPostId
     * 
     * @param type $post_id
     */
    public function findVariationsByPostId($post_id)
    {
        $sql = 'SELECT * FROM ' . TABLE_PREFIX . $this->table . ' post, ' . TABLE_PREFIX . 'postmeta meta 
            INNER JOIN ' . TABLE_PREFIX . 'postmeta metaJoin ON metaJoin.post_id = meta.post_id
            WHERE post.ID = meta.post_id AND post.post_parent = :post_id AND post.post_type = \'product_variation\'
            AND post.post_status = \'publish\' AND meta.meta_key = \'_stock_status\'
            AND meta.meta_value = \'instock\' AND metaJoin.meta_key = \'_stock\'
            AND metaJoin.meta_value != 0';
        $select = $this->prepare($sql);
        $select->bindValue(':post_id', $post_id);
        $select->execute();
        return $select->fetchAll(\PDO::FETCH_ASSOC);
    }

    /**
     * getMinVariationSinglePrice
     * 
     * @param type $post_id
     * @return type
     */
    public function getMinVariationSinglePrice($post_id)
    {
        $sql = 'SELECT MIN(meta_value) as price FROM ' . TABLE_PREFIX . 'postmeta WHERE (post_id = :post_id or post_id IN (SELECT meta.post_id as post_id
            FROM ' . TABLE_PREFIX . $this->table . ' post, wp_postmeta meta
            INNER JOIN wp_postmeta metaJoin ON metaJoin.post_id = meta.post_id
            WHERE post.ID = meta.post_id AND post.post_parent = :post_id AND post.post_type = \'product_variation\'
            AND post.post_status = \'publish\' AND meta.meta_key = \'_stock_status\'
            AND meta.meta_value = \'instock\' AND metaJoin.meta_key = \'_stock\'
            AND metaJoin.meta_value != 0)) and meta_key in (\'_price\', \'_regular_price\', \'_sale_price\') and meta_value != 0';

        $select = $this->prepare($sql);
        $select->bindValue(':post_id', $post_id);
        $select->execute();
        $result = $select->fetch(\PDO::FETCH_ASSOC);
        if (!isset($result['price'])) {
            return false;
        }

        return $result['price'];
    }

    /**
     * getProductVariations
     * 
     * @param type $post_id
     */
    public function getProductVariations($post_id)
    {
        $existing_attributes = array();
        $attributesModel = new WooAttributes();
        $postMeta = new PostMeta();
        $result = $this->findVariationsByPostId($post_id);
        foreach ($result as $item) {
            $metaData = $postMeta->getAttributesByPostId($item['post_id']);
            foreach ($metaData as $metaValues) {
                if (isset($metaValues['meta_key'])) {
                    $name = $attributesModel->getName($metaValues['meta_key']);
                    $existing_attributes[$name['attribute_label']][$item['post_id']] = $metaValues['meta_value'];
                }
            }
            $existing_attributes['images'][$item['post_id']] = $postMeta->getImageByPostId($item['post_id']);
            $existing_attributes['price'][$item['post_id']] = $postMeta->getPriceByPostId($item['post_id']);
        }

        return $existing_attributes;
    }

    /**
     * getGender
     * 
     * @param type $attributes
     * @return string
     */
    public function getGender($attributes)
    {
        if (!$attributes) {
            return 'unisex';
        }

        $params = Settings::maybe_unserialize($attributes);
        if (!isset($params['gender'])) {
            return 'unisex';
        }

        return $params['gender']['value'];
    }

    /**
     * getAgeGroup
     * 
     * @param type $attributes
     * @return string
     */
    public function getAgeGroup($attributes)
    {
        if (!$attributes) {
            return 'adult';
        }

        $params = Settings::maybe_unserialize($attributes);
        if (isset($params['adult'])) {
            return $params['adult']['value'];
        }

        if (isset($params['department_name'])) {
            return $params['department_name']['value'];
        }

        return 'adult';
    }

    /**
     * addVariationPrice
     * 
     * @param type $value
     * @param type $index
     * @param type $priceStash
     * @return type
     */
    public function addVariationPrice($value, $index, $priceStash)
    {
        if (!$value['price'] || isset($priceStash[$index])) {
            $value['price'] = $priceStash[$index];
            $value['price'] = $this->getWooPrice($value, $value['select_currency']);
            return $value;
        }

        $value['price'] = min($priceStash);
        $value['price'] = $this->getWooPrice($value, $value['select_currency']);
        return $value;
    }

    /**
     * addVariationImage
     * 
     * @param type $value
     * @param type $index
     * @param type $imageStash
     * @return type
     */
    public function addVariationImage($value, $index, $imageStash)
    {
        if (isset($imageStash[$index])) {
            $value['image_link'] = $imageStash[$index];
            return $value;
        }

        return $value;
    }

    /**
     * addVariationColor
     * 
     * @param type $value
     * @param type $index
     * @param type $colorStash
     * @return type
     */
    public function addVariationColor($value, $index, $colorStash)
    {
        if (isset($colorStash[$index])) {
            $value['color'] = $colorStash[$index];
            return $value;
        }

        return $value;
    }

    /**
     * addProductVariation
     * 
     * @param type $attribute
     * @param type $value
     * @param type $keyParam
     */
    public function addProductVariation($attribute, $value, $keyParam)
    {
        $publishVariation = $this->checkVariation($keyParam);
        if (!$publishVariation) {
            return false;
        }

        $priceStash = $this->getVariationsPrices();
        $imageStash = $this->getImages();
        $colorStash = $this->getColorProducts();
        $materialStash = $this->getMaterialProducts();
        $sizeStash = $this->getSizeProducts();
        $valueProduct = $this->productValue;
        if (!isset($valueProduct[$attribute])
            || $valueProduct[$attribute] != $value
        ) {
            $valueProduct = $this->addVariationSize($valueProduct, $keyParam, $sizeStash);
            $valueProduct = $this->addVariationPrice($valueProduct, $keyParam, $priceStash);
            $valueProduct = $this->addVariationImage($valueProduct, $keyParam, $imageStash);
            $valueProduct = $this->addVariationColor($valueProduct, $keyParam, $colorStash);
            $valueProduct = $this->addVariationMaterial($valueProduct, $keyParam, $materialStash);
            $tempProduct = $valueProduct;
            $tempProduct[$attribute] = $value;
            $tempProduct['itemId'] = $keyParam;
            $this->addProduct($tempProduct);
        }
    }

    /**
     * getItem
     * 
     * @param type $value
     * @return boolean
     */
    private function getItem($value)
    {
        $item = array();
        $item['id'] = $value['id'];
        $postMeta = PostMeta::instance();
        $item['image_link'] = $postMeta->getImageByPostId($value['id']);
        if (empty($item['image_link'])) {
            return false;
        }

        $item['price'] = $postMeta->getPriceByPostId($value['id']);
        if (empty($item['price'])) {
            return false;
        }

        $item['title'] = $this->getTitle($value['title']);
        $item['description'] = $this->getDescription($value);
        $item['brand'] = WP_ADS_BLOGNAME;
        $item['availability'] = 'in stock';
        $item['link'] = $this->getLink($value['id']);
        $item['condition'] = 'new';
        $item['google_product_category'] = $value['google_product_category'];
        $attributes = $postMeta->getProductAttributesByPostId($value['id']);
        $item['gender'] = $this->getGender($attributes);
        $item['age_group'] = $this->getAgeGroup($attributes);
        return $item;
    }

    /**
     * getWooPrice
     * 
     * @param type $result
     * @param type $currency
     * @return boolean
     */
    private function getWooPrice($result, $currency)
    {
        if (!empty((int)$result['price'])) {
            if ($result['position'] == 'left'
                || $result['position'] == 'left_space'
            ) {
                return $currency . ' ' . $this->priceConvertCurrents($result['price'], $result['default_currency'], $currency);
            } else {
                return $this->priceConvertCurrents($result['price'], $result['default_currency'], $currency) . ' ' . $currency;
            }
        }

        return false;
    }

    /**
     * priceConvertCurrents
     * 
     * @param type $currentPrice
     * @param type $defaultCurrency
     * @param type $currency
     * @return type
     */
    private function priceConvertCurrents($price, $from, $to = 'USD')
    {
        $price = $this->ads_floatvalue($price);
	$args = ADSW_CUVALUE;
        $curValues = $args['value'];
        $curValues['USD'] = 1;

        if (!isset($curValues[$from])
            || !isset($curValues[$to])
        ) {
            return $price;
        }

        $sub = $curValues[$to]/$curValues[$from];
        $price = $price * $sub;
        return round($price, 2);
    }
}